@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: chr.h,v 1.108 2008/01/11 09:12:57 bowkenken Exp $
+* $Id: chr.h,v 1.121 2009/01/31 11:25:27 bowkenken Exp $
 ***************************************************************/
 
 #ifndef	CHR_H
@@ -38,6 +38,7 @@
 #include	"dun.h"
 #include	"fight.h"
 #include	"fx.h"
+#include	"skill.h"
 #include	"misc.h"
 
 #include	"resi.h"
@@ -51,6 +52,11 @@
 * иͤȥ٥
 ***************************************************************/
 
+#define	ABL_SKILL_MAX_N	(ABL_KIND_MAX_N + SKILL_KIND_MAX_N)
+
+/* ٥κǹ */
+#define	ABL_MAX_LEV	9999
+
 /* иͤκ */
 #define	EXP_MAX	99999999
 #define	NEED_EXP_MAX	(EXP_MAX + 1)
@@ -77,7 +83,6 @@
 
 #define	ABL_RATE_MIN_N	((rate_t)0)	/* иͤγ꿶κ */
 #define	ABL_RATE_MAX_N	((rate_t)20)	/* иͤγ꿶κǹ */
-#define	ABL_MAX_LEV	9999	/* ٥κǹ */
 
 /***************************************************************
 * 饯Υե饰
@@ -96,6 +101,7 @@ typedef unsigned long	flg_chr_t;
 #define	FLG_CHR_LIGHT	(1 << 11)	/* äƤ */
 #define	FLG_CHR_DARK	(1 << 12)	/* ǤäƤ */
 #define	FLG_CHR_MAGIC_PROTECT	(1 << 13)	/* ˡ路ʤ */
+#define	FLG_CHR_HYPER	(1 << 14)	/* 緿 */
 #define	FLG_CHR_CAN_DEL	(1 << 20)	/*  */
 #define	FLG_CHR_CAN_DISMISSAL	(1 << 21)	/* Խ */
 
@@ -129,6 +135,8 @@ typedef unsigned long	flg_stat_t;
 #define	FLG_STAT_PASSWALL	(1 << 20)	/* ȴ */
 #define	FLG_STAT_SENSE_INVISIBLE	(1 << 21)	/* ԲĻ봶 */
 
+#define STAT_MAX_N	22
+
 /***************************************************************
 * 饯Υե饰
 * ֤Ѥϥ֡ǡ⽤
@@ -168,7 +176,7 @@ typedef unsigned long	flg_work_t;
 
 /* դĤΨ */
 #define	STAGGER_RATE_CHR_CONFUSION	((rate_t)70)	/*  */
-#define	STAGGER_RATE_CHR_BLIND	((rate_t)80)	/* ܤʤ */
+#define	STAGGER_RATE_CHR_BLIND	((rate_t)30)	/* ܤʤ */
 #define	STAGGER_RATE_CHR_DRUNK	((rate_t)50)	/* äƤ */
 
 /* 󥹥λäƤ륢ƥκ */
@@ -330,8 +338,8 @@ typedef struct {
 
 /* ǽ */
 typedef struct {
-	abl_exp_t	exp[ABL_KIND_MAX_N];
-	abl_lev_t	lev[ABL_KIND_MAX_N];
+	abl_exp_t	exp[ABL_SKILL_MAX_N];
+	abl_lev_t	lev[ABL_SKILL_MAX_N];
 	abl_val_t	hp, mp;
 } abl_t;
 
@@ -392,7 +400,7 @@ typedef struct {
 	rate_t	armor_def, armor_crtcl, armor_ac;
 
 	/* ٥ */
-	long	add_lev[ABL_KIND_MAX_N];
+	long	add_lev[ABL_SKILL_MAX_N];
 	/*  */
 	long	add_resi_n[RESI_KIND_MAX_N];
 	/*  */
@@ -454,8 +462,11 @@ typedef struct {
 
 	/* ǥեȤΥ饹 */
 	class_dflt_t	dflt;
+
 	/* иͤʬ */
 	rate_t	abl_rate[ABL_KIND_MAX_N];
+	/* åȤƤ륹 */
+	skill_kind_t	skill[SKILL_MAX_N];
 } class_t;
 
 /***************************************************************
@@ -499,6 +510,7 @@ typedef enum {
 	ACT_KIND_POTION_FILL,
 	ACT_KIND_SCROLL_READ,
 	ACT_KIND_STICK_ZAP,
+	ACT_KIND_LAMP_RUB,
 	ACT_KIND_FOOD_EAT,
 	ACT_KIND_TOBACCO_DISPOSE,
 	ACT_KIND_SRCH,
@@ -593,6 +605,29 @@ typedef enum {
 	ATTITUDE_ENEMY,
 } attitude_t;
 
+/***************************************************************
+* 饯緿
+***************************************************************/
+
+/* NULL, NORMAL, PERFECT ʳϽ֤¤ѤƤOK */
+
+typedef enum {
+	/* 緿̵ */
+	HYPER_BOSS_NULL,
+	/* ̾ */
+	HYPER_BOSS_NORMAL,
+	/* 緿 */
+	HYPER_BOSS_NW,
+	/* 緿 */
+	HYPER_BOSS_SE,
+	/* 緿 */
+	HYPER_BOSS_NE,
+	/* 緿 */
+	HYPER_BOSS_SW,
+	/* 緿λ */
+	HYPER_BOSS_PERFECT,
+} hyper_boss_t;
+
 /***************************************************************
 * 
 ***************************************************************/
@@ -601,6 +636,11 @@ typedef struct {
 	/* ե饰 */
 	flg_work_t	flg;
 
+	/* 饯緿 */
+	hyper_boss_t	hyper_boss;
+	/* 饯緿ե饰 (GUI) */
+	bool_t	flg_hyper_boss;
+
 	/*  PC ޤƤ륭οͿ */
 	long	caught;
 
@@ -613,6 +653,9 @@ typedef struct {
 	/* ࡦƥݡȤε륫 */
 	long	teleport_n, teleport_party_n;
 
+	/* ˾ߡפμʸȤ뤫? */
+	bool_t	flg_can_wish;
+
 	/* طƤ */
 	struct request_t	*request;
 
@@ -626,6 +669,10 @@ typedef struct {
 
 	/* 󥹥С̿ϵۼȤä? */
 	bool_t	flg_used_energy_drain;
+
+	/* ǤĿ VFX 褹ݤΥ᡼ */
+	long	sync_vfx_num;
+	long	sync_vfx_x, sync_vfx_y;
 } chr_work_t;
 
 /***************************************************************
@@ -795,6 +842,9 @@ typedef struct chr_t {
 	/* եȤü */
 	modifier_t	modifier_fx;
 
+	/* դ */
+	gold_t	contribution;
+
 	/* 󥹥μ */
 	mnstr_kind_t	mnstr_kind, mnstr_kind_org;
 	/* 󥹥ǡơ֥ */
