@@ -9,6 +9,7 @@ void	reset_static_draw( void );
 void	measure_clk_tck( void );
 void	print_wrn( const char *s, ... );
 void	print_err( const char *s, ... );
+void	print_lex_wrn( const char *s, ... );
 void	print_lex_err( const char *s, ... );
 void	print_words(
 	const char *name,
@@ -17,7 +18,21 @@ void	print_words(
 	const char *fmt_male,
 	...
 );
-char	*get_words_line( char *p, long max_len );
+void	print_words_win(
+	const char *name,
+	sex_t sex,
+	const char *fmt_female,
+	const char *fmt_male,
+	...
+);
+void	print_words_sub(
+	bool_t flg_draw_win,
+	const char *name,
+	const char *buf
+);
+const char	*get_words_line( const char *p, long draw_max_n );
+const char	*get_next_char_str( const char *p );
+long	get_next_char_len( const char *p );
 void	print_msg_mbr( chr_t *chr, flg_msg_t flg, const char *s, ... );
 void	print_msg_find( chr_t *chr, flg_msg_t flg, const char *s, ... );
 void	print_msg( flg_msg_t flg, const char *s, ... );
@@ -35,6 +50,9 @@ long	get_ls_msg_n( void );
 void	draw_ls_msg( long n );
 ls_msg_t	*get_ls_msg( long idx );
 void	print_str( long scrn_x, long scrn_y, const char *s, ... );
+bool_t	print_vfx( long scrn_x, long scrn_y, const char *s, ... );
+bool_t	print_vfx_cui( long scrn_x, long scrn_y, const char *s, ... );
+void	wipe_vfx( long scrn_x, long scrn_y, long scrn_w, long scrn_h );
 bool_t	set_redraw_every_turn( bool_t flg );
 bool_t	chk_redraw_every_turn( void );
 void	wipe_all( void );
@@ -47,8 +65,10 @@ void	draw_map_sub(
 	bool_t flgForce
 );
 void	redraw_map( void );
+void	redraw_map_force( void );
+void	redraw_map_sub( bool_t flgForce );
 void	wipe_map( void );
-void	force_redraw_map( void );
+void	force_clr_redraw_map( void );
 void	draw_party( void );
 void	draw_mbr( mbr_t *p );
 void	draw_square( void );
@@ -99,24 +119,50 @@ void	vfx_arw_mul_line(
 	long frame, long clr_frame, bool_t flg_clr
 );
 void	vfx_num( chr_t *p, long n );
-void	draw_lev_up( long map_x, long map_y, long n, chr_t *chr );
+void	bgn_sync_vfx_num( void );
+void	end_sync_vfx_num( void );
+long	get_draw_num_frame( long len );
+long	get_draw_num_sync_max_len( void );
+bool_t	draw_num_sync( long idx );
 void	draw_num( long map_x, long map_y, long n, chr_t *chr );
+void	draw_num_x_1(
+	long map_x, long map_y,
+	long n, long idx,
+	chr_t *chr
+);
+void	draw_lev_up( long map_x, long map_y, long n, chr_t *chr );
 void	draw_blast(
 	chr_kind_t kind,
 	long map_x, long map_y,
 	blast_t *p, char *str
 );
+void	draw_blast_cui(
+	chr_kind_t kind,
+	long map_x, long map_y,
+	blast_t *p, char *str
+);
+void	draw_blast_sub(
+	bool_t flg_cui,
+	chr_kind_t kind,
+	long map_x, long map_y,
+	blast_t *p, char *str
+);
 void	draw_request( request_t *req );
+void	draw_words( const char *name, const char *words, ... );
+void	redraw_mbr_data( mbr_t *org, mbr_t *mbr, long page_n );
 void	draw_mbr_data_all( mbr_t *org, mbr_t *mbr, long page_n );
+void	draw_mbr_data_all_cui( mbr_t *org, mbr_t *mbr, long page_n );
 void	draw_mbr_data( mbr_t *org, mbr_t *mbr, data_n_t n, bool_t flg_rev );
 data_n_t	next_mbr_data(
 	long page_n, data_n_t n, long d,
 	mbr_data_edit_kind_t edit_kind
 );
+bool_t	chk_sel_data_n( data_n_t n, mbr_data_edit_kind_t edit_kind );
 bool_t	gets_mbr_name( mbr_t *mbr );
 bool_t	gets_mbr_face( mbr_t *mbr );
 bool_t	gets_race_name( mbr_t *mbr );
 bool_t	gets_pet_name( pet_t *pet );
+void	print_skill_hint( long x, long y, skill_kind_t kind );
 void	draw_edit_class_all(
 	class_t *org, class_t *dst,
 	edit_class_t sel_n, bool_t flg_sel_add
@@ -125,7 +171,16 @@ void	draw_edit_class(
 	class_t *org, class_t *dst, edit_class_t n,
 	bool_t flg_sel, bool_t flg_sel_add
 );
+void	draw_edit_skill_all(
+	class_t *org, class_t *dst,
+	edit_skill_t sel_n
+);
+void	draw_edit_skill(
+	class_t *org, class_t *dst, edit_skill_t n,
+	bool_t flg_sel
+);
 bool_t	gets_class_name( char *name );
+bool_t	gets_item_alias( char *name );
 bool_t	gets_auto_mark_info( char *name );
 bool_t	gets_auto_mark_hint( char *name );
 bool_t	gets_line( const char *prompt, char *s, long max_len );
@@ -141,6 +196,7 @@ void	reset_crsr( void );
 void	draw_crsr( void );
 void	draw_sub_crsr( void );
 pos_t	*get_map_draw_pos( void );
+pos_t	*get_map_draw_pos_gui( void );
 rate_t	get_vfx_wait( void );
 rate_t	set_vfx_wait( rate_t n );
 flg_vfx_mbr_t	get_flg_vfx_mbr( void );
@@ -148,6 +204,10 @@ flg_vfx_mbr_t	set_flg_vfx_mbr( flg_vfx_mbr_t flg );
 flg_vfx_mnstr_t	get_flg_vfx_mnstr( void );
 flg_vfx_mnstr_t	set_flg_vfx_mnstr( flg_vfx_mnstr_t flg );
 void	wait_clock( long frame );
+void	wait_frame( long frame );
+void	wait_100_n_msec( long n );
+void	wait_100_msec( void );
+void	sleep_msec( long n );
 void	set_flg_break_key( bool_t flg_break );
 bool_t	chk_flg_break_key( void );
 long	read_char( long *p );
@@ -175,10 +235,14 @@ long	get_key_half_wait( bool_t *p_flg_cont_exec );
 long	get_key_std( bool_t *p_flg_cont_exec );
 long	get_key_no_clr( bool_t *p_flg_cont_exec );
 long	get_key_no_clr_default( bool_t *p_flg_cont_exec );
+long	get_key_no_wait( void );
 bool_t	chk_key_buf( void );
 long	get_key_buf_char( void );
 long	get_key_buf_char_sub( void );
-bool_t	set_key_buf_str( const char *s );
+bool_t	set_key_buf_char_head( long c );
+bool_t	set_key_buf_char_tail( long c );
+bool_t	set_key_buf_str_head( const char *s );
+bool_t	set_key_buf_str_tail( const char *s );
 long	get_key_tab_n( const char *key );
 char	*get_key_tab_cmd( const char *key );
 char	*get_key_tab_cmd_char( long c );
