@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mmenu.c,v 1.43 2008/10/07 02:59:19 bowkenken Exp $
+* $Id: mmenu.c,v 1.48 2009/02/01 23:45:18 bowkenken Exp $
 ***************************************************************/
 
 #define	MMENU_C
@@ -128,7 +128,8 @@ void	reset_static_main_menu( void )
 /**/
 
 char	*exec_menu_sel_mbr(
-	mbr_t **mbr, char *ttl, bool_t flg_sel_dead, bool_t flg_sel_pet
+	mbr_t **mbr, char *ttl,
+	bool_t flg_sel_dead, bool_t flg_sel_pet
 )
 {
 	party_t	*pty = get_party();
@@ -776,6 +777,10 @@ char	*cb_menu_sel_obj_init( menu_t **pp, menu_t **dflt )
 	if( chk_flg( flg_chr, FLG_MAP_CHR_INVISIBLE ) )
 		set_flg_menu( *pp, "monster", FLG_MENU_HIDDEN );
 
+	/* 饹ܥ */
+	if( chk_xx_pos( g_crsr.x, g_crsr.y ) )
+		clr_flg_menu( *pp, "monster", FLG_MENU_HIDDEN );
+
 	/* ƥ */
 	if( get_item( g_crsr.x, g_crsr.y ) == NULL )
 		set_flg_menu( *pp, "item", FLG_MENU_HIDDEN );
@@ -1421,6 +1426,8 @@ char	*cb_menu_item( menu_t **pp, menu_t **dflt )
 		g_sel_mbr_n = pre_sel_mbr_n;
 	else
 		pre_sel_mbr_n = g_sel_mbr_n;
+	static_cb_menu_item__pre_sel_mbr_n = pre_sel_mbr_n;
+
 	if( g_sel_mbr_n <= -1 )
 		return STR_MENU_ERR;
 	if( g_sel_mbr_n >= MBR_MAX_N )
@@ -1634,6 +1641,7 @@ char	*cb_menu_item_sel_init( menu_t **pp, menu_t **dflt )
 	clr_flg_menu( *pp, "send", FLG_MENU_HIDDEN );
 	clr_flg_menu( *pp, "receive", FLG_MENU_HIDDEN );
 	clr_flg_menu( *pp, "drop", FLG_MENU_HIDDEN );
+	clr_flg_menu( *pp, "alias", FLG_MENU_HIDDEN );
 	clr_flg_menu( *pp, "check", FLG_MENU_HIDDEN );
 
 	switch( g_sel_item->kind ){
@@ -1997,6 +2005,18 @@ char	*cb_menu_item_drop( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_item_alias( menu_t **pp, menu_t **dflt )
+{
+	if( g_sel_item == NULL )
+		return STR_MENU_ERR;
+
+	set_item_alias( g_sel_item );
+
+	return STR_MENU_CANCEL;
+}
+
+/**/
+
 char	*cb_menu_item_chk( menu_t **pp, menu_t **dflt )
 {
 	if( g_sel_item == NULL )
@@ -2583,7 +2603,7 @@ char	*cb_menu_save_wiz( menu_t **pp, menu_t **dflt )
 
 void	menu_load_wiz( void )
 {
-	if( menu_load( TRUE ) )
+	if( menu_load( TRUE, FALSE ) )
 		reset_all_save( TRUE );
 }
 
@@ -2597,13 +2617,13 @@ void	menu_save_wiz( void )
 
 /**/
 
-bool_t	menu_load( bool_t flg_txt )
+bool_t	menu_load( bool_t flg_txt, bool_t flg_new_game )
 {
 	long	n;
 	ask_t	ask;
 
-	n = menu_sel_save_load( FALSE, flg_txt );
-	if( n <= -1 )
+	n = menu_sel_save_load( FALSE, flg_txt, flg_new_game );
+	if( n == SAVE_N_SEL )
 		return FALSE;
 
 	ask = exec_menu_ask( MSG_TTL_MENU_ASK_LOAD, ASK_NO, TRUE );
@@ -2644,7 +2664,7 @@ bool_t	menu_save_n( bool_t flg_txt, long save_n )
 	}
 
 	if( save_n == SAVE_N_SEL ){
-		save_n = menu_sel_save_load( TRUE, flg_txt );
+		save_n = menu_sel_save_load( TRUE, flg_txt, FALSE );
 
 		if( save_n == SAVE_N_AUTO ){
 			print_msg( FLG_MSG_ERR,
@@ -2679,7 +2699,9 @@ bool_t	menu_save_n( bool_t flg_txt, long save_n )
 
 /**/
 
-long	menu_sel_save_load( bool_t flg_save, bool_t flg_txt )
+long	menu_sel_save_load(
+	bool_t flg_save, bool_t flg_txt, bool_t flg_new_game
+)
 {
 	menu_t	*ls;
 	menu_t	*dflt;
@@ -2706,8 +2728,11 @@ long	menu_sel_save_load( bool_t flg_save, bool_t flg_txt )
 
 	for( i = 0; i < MENU_SAVE_LOAD_MAX_N; i++ ){
 		bool_t	flg_exist_data;
+		bool_t	flg_page;
 
-		if( (i != 0) && (i % MENU_SAVE_LOAD_N_PER_PAGE == 0) ){
+		flg_page = ((i % MENU_SAVE_LOAD_N_PER_PAGE) == 0);
+
+		if( flg_page && (i != 0) ){
 			key = 'A';
 
 			str_nz_cpy( ls[k].path,
@@ -2725,12 +2750,28 @@ long	menu_sel_save_load( bool_t flg_save, bool_t flg_txt )
 			k++;
 		}
 
-		if( i % MENU_SAVE_LOAD_N_PER_PAGE == 0 ){
+		if( flg_page ){
 			set_ttl_menu_save_load( &(ls[k]),
 					flg_save, flg_txt );
 			k++;
 		}
 
+		if( flg_new_game && (i == 0) ){
+			str_nz_cpy( ls[k].path,
+					"/save load list/new game",
+					MENU_PATH_MAX_LEN );
+			str_nz_cpy( ls[k].kind, "<Item>",
+					MENU_KIND_MAX_LEN );
+			ls[k].accel_key = 'Z';
+			ls[k].short_key[0] = '\0';
+			ls[k].n_msg = N_MSG_SAVE_LOAD_TTL_NEW_GAME;
+			ls[k].msg[0] = '\0';
+			ls[k].call_back = cb_menu_new_game;
+			ls[k].arg = NULL;
+			ls[k].flg = FLG_NULL;
+			k++;
+		}
+
 		sn_printf( ls[k].path, MENU_PATH_MAX_LEN,
 				"/save load list/%ld", i );
 		str_nz_cpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
@@ -2751,16 +2792,20 @@ long	menu_sel_save_load( bool_t flg_save, bool_t flg_txt )
 	}
 	ls[k].path[0] = '\0';
 
-	g_sel_save_load_n = -1;
+	g_sel_save_load_n = SAVE_N_SEL;
 
 	while( 1 ){
 		ret = exec_menu( ls, dflt, NULL );
 
 		if( ret == NULL ){
-			if( !flg_save && (g_sel_save_load_n <= -1) )
-				continue;
-			else
+			if( g_sel_save_load_n == SAVE_N_SEL ){
+				if( flg_new_game )
+					return SAVE_N_SEL;
+				if( !flg_save )
+					continue;
+			} else {
 				return g_sel_save_load_n;
+			}
 		}
 
 		return SAVE_N_SEL;
@@ -2817,6 +2862,13 @@ char	*cb_menu_save_load( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_new_game( menu_t **pp, menu_t **dflt )
+{
+	return NULL;
+}
+
+/**/
+
 void	set_flg_no_save( bool_t flg )
 {
 	g_flg_no_save = flg;
