@@ -15,6 +15,8 @@ bool_t	scene_srch_boss( void );
 bool_t	scene_srch_battle( void );
 scene_n_t	change_scene( scene_n_t n );
 scene_n_t	get_scene( void );
+last_scene_n_t	change_last_scene( last_scene_n_t n );
+last_scene_n_t	get_last_scene( void );
 bool_t	chk_scene_group( scene_group_n_t group_n );
 
 #ifdef __cplusplus
