@@ -1,6 +1,6 @@
 /***************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2008 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2009 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
 *   URL: https://sourceforge.jp/projects/lnl/
@@ -25,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: tmenu.c,v 1.127 2008/09/21 00:58:16 bowkenken Exp $
+* $Id: tmenu.c,v 1.141 2009/02/19 03:42:08 bowkenken Exp $
 ***************************************************************/
 
 #define	TMENU_C
@@ -53,7 +53,7 @@
 		/ LS_MBR_MAX_N_PER_PAGE)
 
 #define	ORDER_MILK_MAX_LEV	100
-#define	ORDER_MILK_MIN_RATE	((rate_t)1)
+#define	ORDER_MILK_MIN_RATE	((rate_t)10)
 #define	ORDER_MILK_MAX_RATE	((rate_t)_100_PERCENT)
 
 /**/
@@ -74,6 +74,10 @@ static menu_t	*g_navi_dflt;
 /**/
 
 static edit_class_t	static_exec_edit_class__n;
+static edit_skill_t	static_exec_edit_skill__n;
+
+skill_kind_t	g_sel_skill_kind;
+edit_skill_t	g_skill_slot_n;
 
 /**/
 
@@ -88,6 +92,9 @@ void	init_town_menu( void )
 	g_sel_class_n = 0;
 
 	g_navi_dflt = NULL;
+
+	g_sel_skill_kind = SKILL_KIND_NULL;
+	g_skill_slot_n = EDIT_SKILL_NEXT;
 }
 
 /***************************************************************
@@ -115,6 +122,7 @@ void	reset_static_town_menu( void )
 	/**/
 
 	static_exec_edit_class__n = EDIT_CLASS_NEXT;
+	static_exec_edit_skill__n = EDIT_SKILL_NEXT;
 }
 
 /**/
@@ -468,6 +476,8 @@ char	*menu_order( void )
 		g_mbr_order[i] = NULL;
 
 	while( 1 ){
+		next_g_sel_mbr( FALSE );
+
 		str_goto = menu_order_sel_mbr();
 		if( str_goto != NULL ){
 			if( strcmp( str_goto, STR_MENU_ERR ) == 0 )
@@ -549,6 +559,9 @@ char	*menu_order_sel_mbr( void )
 		mbr_n++;
 	}
 
+	if( g_sel_mbr == NULL )
+		dflt = &(p[k]);
+
 	str_nz_cpy( p[k].path, "/order/chose", MENU_PATH_MAX_LEN );
 	str_nz_cpy( p[k].kind, "<Item>", MENU_KIND_MAX_LEN );
 	p[k].accel_key = 'Z';
@@ -578,18 +591,52 @@ char	*cb_menu_order_sel( menu_t **pp, menu_t **dflt )
 	g_sel_mbr = (mbr_t *)((*pp)->arg);
 
 	str_goto = menu_shop( g_shop_n, g_sel_mbr, NULL );
-	if( strcmp( str_goto, STR_MENU_CANCEL ) == 0 )
-		return STR_MENU_CANCEL;
+	if( str_goto != NULL )
+		if( strcmp( str_goto, STR_MENU_CANCEL ) == 0 )
+			return STR_MENU_CANCEL;
 
 	if( g_sel_mbr == NULL )
 		return STR_MENU_ERR;
-	g_mbr_order[g_sel_mbr->mbr_n] = g_sel_art;
 
+	g_mbr_order[g_sel_mbr->mbr_n] = g_sel_art;
 	return str_goto;
 }
 
 /**/
 
+void	next_g_sel_mbr( bool_t flg_loop )
+{
+	party_t	*pty;
+	long	n, m;
+	long	i;
+
+	pty = get_party();
+	if( g_sel_mbr == NULL )
+		n = 0;
+	else
+		n = g_sel_mbr->mbr_n + 1;
+
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		m = n + i;
+		if( !flg_loop && (m >= MBR_MAX_N) )
+			break;
+		m %= MBR_MAX_N;
+
+		if( chk_flg_or( pty->mbr[m]->stat,
+				FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD ) ){
+			continue;
+		}
+
+		g_sel_mbr = pty->mbr[m];
+		return;
+	}
+
+	g_sel_mbr = NULL;
+}
+
+/**/
+
 char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt )
 {
 	party_t	*pty = get_party();
@@ -615,6 +662,7 @@ char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt )
 			continue;
 
 		add_chr_gold( pty->mbr[i], -price );
+		call_game_sound_play( SOUND_KIND_SHOP_DRINK, 2 );
 
 		tmp = alloc_item();
 		if( tmp == NULL )
@@ -1106,6 +1154,7 @@ char	*cb_menu_shop( menu_t **pp, menu_t **dflt )
 
 		add_chr_gold( g_sel_mbr, +(price * deal_n / n) );
 	}
+	call_game_sound_play( SOUND_KIND_SHOP_PAY, 1 );
 
 	return "/shop";
 }
@@ -1156,7 +1205,7 @@ long	menu_shop_buy(
 		return 0;
 
 	/* 㤦ǧ */
-	ask = exec_menu_ask( MSG_TTL_MENU_BUY, ASK_NO, TRUE );
+	ask = exec_menu_ask_no_msg( MSG_TTL_MENU_BUY, ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
 		break;
@@ -1228,7 +1277,8 @@ long	menu_shop_sale(
 
 		/* 뤫ǧ */
 
-		ask = exec_menu_ask( MSG_TTL_MENU_SALE, ASK_NO, TRUE );
+		ask = exec_menu_ask_no_msg( MSG_TTL_MENU_SALE,
+				ASK_NO, TRUE );
 		switch( ask ){
 		case ASK_YES:
 			break;
@@ -2095,6 +2145,8 @@ char	*cb_menu_bar_arrange_1( long d )
 
 	arrange_cast_tab_1( trgt_ls_mbr_n, d );
 	arrange_auto_mark_mbr_1( trgt_ls_mbr_n, d );
+	arrange_item_1( trgt_ls_mbr_n, d );
+	arrange_equip_1( trgt_ls_mbr_n, d );
 
 	/**/
 
@@ -2126,6 +2178,8 @@ char	*cb_menu_bar_arrange( long d )
 
 	arrange_cast_tab( trgt_ls_mbr_n, d );
 	arrange_auto_mark_mbr( trgt_ls_mbr_n, d );
+	arrange_item( trgt_ls_mbr_n, d );
+	arrange_equip( trgt_ls_mbr_n, d );
 
 	tmp_mbr = g_sel_ls_mbr;
 	if( d <= -1 ){
@@ -2198,8 +2252,8 @@ bool_t	exec_menu_new_mbr( void )
 		menu_mbr_data( ls_mbr[n], &page_n, &data_n,
 				MBR_DATA_EDIT_KIND_NEW );
 
-		ask = exec_menu_ask(
-				MSG_TTL_MENU_ASK_NEW_MBR, ASK_NO, TRUE );
+		ask = exec_menu_ask( MSG_TTL_MENU_ASK_NEW_MBR,
+				ASK_NO, TRUE );
 		if( (ask == ASK_YES) || (ask == ASK_NO) )
 			break;
 		if( ask == ASK_ERR )
@@ -2447,10 +2501,59 @@ char	*cb_menu_bar_arrange_bottom_class( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_bar_edit_skill( menu_t **pp, menu_t **dflt )
+{
+	char	*str_goto;
+
+	while( 1 ){
+		str_goto = exec_menu_class( &g_sel_class_n );
+		if( str_goto != NULL )
+			return str_goto;
+
+		menu_edit_skill( g_sel_class_n );
+	}
+}
+
+/**/
+
+char	*menu_edit_skill( long n )
+{
+	class_t	*p;
+	long	add;
+
+	p = get_class_tab();
+	if( p == NULL )
+		return STR_MENU_ERR;
+
+	while( 1 ){
+		add = exec_edit_skill( &(p[n]), FALSE );
+		if( add == 0 )
+			break;
+
+		if( add > 0 ){
+			n++;
+			if( p[n].name[0] == '\0' )
+				n = 0;
+		} else {
+			n--;
+			if( n < 0 ){
+				for( n = 0; n < CLASS_MAX_N; n++ )
+					if( p[n + 1].name[0] == '\0' )
+						break;
+			}
+		}
+	}
+
+	return NULL;
+}
+
+/**/
+
 char	*cb_menu_bar_new_class( menu_t **pp, menu_t **dflt )
 {
 	class_t	*p;
 	long	n;
+	long	i;
 
 	p = get_class_tab();
 	if( p == NULL )
@@ -2465,6 +2568,10 @@ char	*cb_menu_bar_new_class( menu_t **pp, menu_t **dflt )
 	}
 
 	str_nz_cpy( p[n].name, MSG_MENU_NEW_CLASS, CLASS_NAME_MAX_LEN );
+	for( i = 0; i < ABL_KIND_MAX_N; i++ )
+		p[n].abl_rate[i] = (rate_t)0;
+	for( i = 0; i < SKILL_MAX_N; i++ )
+		p[n].skill[i] = SKILL_KIND_NULL;
 
 	menu_new_class( n );
 
@@ -2531,6 +2638,7 @@ long	exec_edit_class( class_t *org, bool_t flg_new )
 			if( exec_menu_ask_edit_class(
 					org, work, flg_new ) ){
 				call_gui_class_edit_wipe();
+				static_exec_edit_class__n = n;
 				return 0;
 			}
 			draw_edit_class_all( org, work, n, flg_sel_add );
@@ -2539,6 +2647,7 @@ long	exec_edit_class( class_t *org, bool_t flg_new )
 			if( n == EDIT_CLASS_NEXT ){
 				if( exec_menu_ask_edit_class(
 						org, work, flg_new ) ){
+					static_exec_edit_class__n = n;
 					return +1;
 				}
 				draw_edit_class_all( org, work, n,
@@ -2547,6 +2656,7 @@ long	exec_edit_class( class_t *org, bool_t flg_new )
 			} else if( n == EDIT_CLASS_PREV ){
 				if( exec_menu_ask_edit_class(
 						org, work, flg_new ) ){
+					static_exec_edit_class__n = n;
 					return -1;
 				}
 				draw_edit_class_all( org, work, n,
@@ -2578,16 +2688,280 @@ long	exec_edit_class( class_t *org, bool_t flg_new )
 		case CASE_CRSR_UP:
 			n = (edit_class_t)(n - 1);
 			break;
+		case 'g':
+			exec_back_log();
+			break;
 		default:
 			break;
 		}
 	}
 
+	static_exec_edit_class__n = n;
 	return 0;
 }
 
 /**/
 
+long	exec_edit_skill( class_t *org, bool_t flg_new )
+{
+	edit_skill_t	*pn, prev_n;
+	class_t	edit, *work;
+	long	slot;
+
+	g_skill_slot_n = static_exec_edit_skill__n;
+
+	if( org == NULL )
+		return 0;
+
+	work = &edit;
+	*work = *org;
+
+	pn = &g_skill_slot_n;
+	prev_n = *pn;
+
+	draw_edit_skill_all( org, work, *pn );
+
+	while( 1 ){
+		if( *pn > EDIT_SKILL_MAX_N - 1 )
+			*pn = (edit_skill_t)(EDIT_SKILL_NULL + 1);
+		if( *pn < EDIT_SKILL_NULL + 1 )
+			*pn = (edit_skill_t)(EDIT_SKILL_MAX_N - 1);
+
+		draw_edit_skill( org, work, prev_n, FALSE );
+		draw_edit_skill( org, work, *pn, TRUE );
+		prev_n = *pn;
+
+		call_gui_skill_edit_draw( work );
+
+		switch( get_key( g_flg_clr_key_buf_menu ) ){
+		case CASE_CANCEL:
+		case CASE_EXEC:
+			if( exec_menu_ask_edit_skill(
+					org, work, flg_new ) ){
+				call_gui_skill_edit_wipe();
+				static_exec_edit_skill__n = *pn;
+				return 0;
+			}
+			draw_edit_skill_all( org, work, *pn );
+			break;
+		case CASE_ENTER:
+			if( *pn == EDIT_SKILL_NEXT ){
+				if( exec_menu_ask_edit_skill(
+						org, work, flg_new ) ){
+					static_exec_edit_skill__n = *pn;
+					return +1;
+				}
+				draw_edit_skill_all( org, work, *pn );
+				break;
+			} else if( *pn == EDIT_SKILL_PREV ){
+				if( exec_menu_ask_edit_skill(
+						org, work, flg_new ) ){
+					static_exec_edit_skill__n = *pn;
+					return -1;
+				}
+				draw_edit_skill_all( org, work, *pn );
+				break;
+			} else if( *pn == EDIT_SKILL_CLASS_NAME ){
+				draw_edit_skill( org, work, *pn, FALSE );
+				gets_class_name( work->name );
+				break;
+			} else if( (EDIT_SKILL_A_00 <= *pn)
+					&& (*pn <= EDIT_SKILL_A_09) ){
+				slot = *pn - EDIT_SKILL_A_00;
+
+				exec_sel_skill_edit( work, slot );
+				draw_edit_skill_all( org, work, *pn );
+				break;
+			} else if( (EDIT_SKILL_B_00 <= *pn)
+					&& (*pn <= EDIT_SKILL_B_09) ){
+				slot = *pn - EDIT_SKILL_B_00
+						+ SKILL_PER_PAGE_MAX_N;
+
+				exec_sel_skill_edit( work, slot );
+				draw_edit_skill_all( org, work, *pn );
+				break;
+			}
+			break;
+		case CASE_CRSR_LEFT:
+		case CASE_CRSR_RIGHT:
+			break;
+		case CASE_CRSR_DOWN:
+			*pn = (edit_skill_t)(*pn + 1);
+			break;
+		case CASE_CRSR_UP:
+			*pn = (edit_skill_t)(*pn - 1);
+			break;
+		case 'g':
+			exec_back_log();
+			break;
+		default:
+			break;
+		}
+	}
+
+	static_exec_edit_skill__n = *pn;
+	return 0;
+}
+
+/**/
+
+void	set_edit_skill_n( edit_skill_t n )
+{
+	g_skill_slot_n = n;
+}
+
+/**/
+
+edit_skill_t	get_edit_skill_n( void )
+{
+	return g_skill_slot_n;
+}
+
+/**/
+
+char	*exec_sel_skill_edit( class_t *work, long slot )
+{
+	char	*ret;
+	long	i;
+
+	if( work == NULL )
+		return NULL;
+	if( slot < 0 )
+		return NULL;
+	if( slot >= SKILL_MAX_N )
+		return NULL;
+
+	ret = exec_sel_skill( &(work->skill[slot]) );
+
+	if( ret != NULL )
+		return ret;
+	if( work->skill[slot] == SKILL_KIND_NULL )
+		return NULL;
+
+	for( i = 0; i < SKILL_MAX_N; i++ ){
+		if( i == slot )
+			continue;
+
+		if( work->skill[i] == work->skill[slot] )
+			work->skill[i] = SKILL_KIND_NULL;
+	}
+
+	return NULL;
+}
+
+/**/
+
+char	*exec_sel_skill( skill_kind_t *ret_kind )
+{
+	long	i, j, k;
+	menu_t	*p, *dflt;
+	skill_kind_t	kind;
+	char	*ret;
+
+	p = ls_menu_sel_skill;
+	dflt = NULL;
+
+	k = 0;
+	str_nz_cpy( p[k].path, "/select skill",
+			MENU_PATH_MAX_LEN );
+	str_nz_cpy( p[k].kind, "<Branch>",
+			MENU_KIND_MAX_LEN );
+	p[k].accel_key = '\0';
+	p[k].short_key[0] = '\0';
+	p[k].msg[0] = '\0';
+	p[k].call_back = NULL;
+	p[k].arg = NULL;
+	p[k].flg = FLG_NULL;
+	k++;
+
+	for( i = SKILL_GROUP_KIND_NULL + 1;
+			i < SKILL_GROUP_KIND_MAX_N; i++ ){
+		str_nz_cpy( p[k].path, "/select skill/--",
+				MENU_PATH_MAX_LEN );
+		str_nz_cpy( p[k].kind, "<Title>",
+				MENU_KIND_MAX_LEN );
+		p[k].accel_key = '\0';
+		p[k].short_key[0] = '\0';
+		str_nz_cpy( p[k].msg,
+				get_skill_group_name( (skill_group_kind_t)i ),
+				MENU_MSG_MAX_LEN );
+		p[k].call_back = NULL;
+		p[k].arg = NULL;
+		p[k].flg = FLG_NULL;
+		k++;
+
+		kind = SKILL_KIND_NULL;
+		sn_printf( p[k].path, MENU_PATH_MAX_LEN,
+				"/select skill/%ld", k );
+		str_nz_cpy( p[k].kind, "<Item>",
+				MENU_KIND_MAX_LEN );
+		p[k].accel_key = 'Z';
+		p[k].short_key[0] = '\0';
+		str_nz_cpy( p[k].msg,
+				get_skill_name_from_kind( kind ),
+				MENU_MSG_MAX_LEN );
+		p[k].call_back = cb_menu_sel_skill;
+		p[k].arg = (void *)kind;
+		p[k].flg = FLG_NULL;
+		k++;
+
+		for( j = 0; j < SKILL_PER_GROUP_MAX_N; j++ ){
+			kind = get_skill_kind_from_group(
+					(skill_group_kind_t)i, j );
+			if( kind == SKILL_KIND_NULL )
+				break;
+
+			if( kind == g_sel_skill_kind )
+				dflt = &(p[k]);
+
+			sn_printf( p[k].path, MENU_PATH_MAX_LEN,
+					"/select skill/%ld", k );
+			str_nz_cpy( p[k].kind, "<Item>",
+					MENU_KIND_MAX_LEN );
+			p[k].accel_key = (char)('A' + j);
+			p[k].short_key[0] = '\0';
+			str_nz_cpy( p[k].msg,
+					get_skill_name_from_kind( kind ),
+					MENU_MSG_MAX_LEN );
+			p[k].call_back = cb_menu_sel_skill;
+			p[k].arg = (void *)kind;
+			p[k].flg = FLG_NULL;
+			k++;
+		}
+
+		str_nz_cpy( p[k].path, "/select skill/--",
+				MENU_PATH_MAX_LEN );
+		str_nz_cpy( p[k].kind, "<PageSeparator>",
+				MENU_KIND_MAX_LEN );
+		p[k].accel_key = '\0';
+		p[k].short_key[0] = '\0';
+		p[k].msg[0] = '\0';
+		p[k].call_back = NULL;
+		p[k].arg = NULL;
+		p[k].flg = FLG_NULL;
+		k++;
+	}
+	p[k].path[0] = '\0';
+
+	ret = exec_menu( p, dflt, NULL );
+	if( ret != NULL )
+		return ret;
+
+	if( ret_kind != NULL )
+		*ret_kind = g_sel_skill_kind;
+	return NULL;
+}
+
+/**/
+
+char	*cb_menu_sel_skill( menu_t **pp, menu_t **dflt )
+{
+	g_sel_skill_kind = (skill_kind_t)(long)((*pp)->arg);
+	return NULL;
+}
+
+/**/
+
 char	*cb_menu_bar_remove_class( menu_t **pp, menu_t **dflt )
 {
 	long	class_n;
@@ -2674,6 +3048,33 @@ bool_t	exec_menu_ask_edit_class(
 
 /**/
 
+bool_t	exec_menu_ask_edit_skill(
+	class_t *org, class_t *work, bool_t flg_new
+)
+{
+	ask_t	ask;
+
+	if( cmp_class( org, work ) )
+		return TRUE;
+
+	ask = exec_menu_ask( MSG_TTL_MENU_ASK_EDIT_CLASS, ASK_NO, TRUE );
+	switch( ask ){
+	case ASK_YES:
+		*org = *work;
+		break;
+	case ASK_NO:
+		*work = *org;
+		break;
+	case ASK_CANCEL:
+	case ASK_ERR:
+		return FALSE;	/* Cancel */
+	}
+
+	return TRUE;
+}
+
+/**/
+
 void	menu_edit_class_inc_abl(
 	class_t *p, edit_class_t n, long add
 )
@@ -2789,9 +3190,17 @@ char	*cb_menu_save_bar( menu_t **pp, menu_t **dflt )
 
 /**/
 
+void	menu_load_start( void )
+{
+	if( menu_load( FALSE, TRUE ) )
+		reset_all_save( FALSE );
+}
+
+/**/
+
 void	menu_load_bar( void )
 {
-	if( menu_load( FALSE ) )
+	if( menu_load( FALSE, FALSE ) )
 		reset_all_save( FALSE );
 }
 
@@ -3075,7 +3484,7 @@ char	*cb_menu_magic_guild( menu_t **pp, menu_t **dflt )
 	print_msg( FLG_NULL, MSG_MENU_BUY, tab->name );
 
 	/* 㤦ǧ */
-	ask = exec_menu_ask( MSG_TTL_MENU_BUY, ASK_NO, TRUE );
+	ask = exec_menu_ask_no_msg( MSG_TTL_MENU_BUY, ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
 		break;
@@ -3086,6 +3495,7 @@ char	*cb_menu_magic_guild( menu_t **pp, menu_t **dflt )
 	}
 
 	add_chr_gold( g_sel_mbr, -get_price_spell( kind ) );
+	call_game_sound_play( SOUND_KIND_SHOP_PAY, 1 );
 	set_chr_learned_spell( g_sel_mbr, kind, TRUE );
 
 	return "/magic guild";
@@ -3579,7 +3989,7 @@ char	*cb_menu_pet_shop_remove_sel( menu_t **pp, menu_t **dflt )
 	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_ASK_REMOVE, pet->name );
 	curs_attrset_dflt();
 
-	ask = exec_menu_ask( MSG_TTL_MENU_PET_SHOP_ASK_REMOVE,
+	ask = exec_menu_ask_no_msg( MSG_TTL_MENU_PET_SHOP_ASK_REMOVE,
 			ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
@@ -3717,7 +4127,7 @@ char	*cb_menu_pet_shop_buy_sel( menu_t **pp, menu_t **dflt )
 	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_ASK_BUY,
 			get_mnstr_name( kind ) );
 
-	ask = exec_menu_ask( MSG_TTL_MENU_PET_SHOP_ASK_BUY,
+	ask = exec_menu_ask_no_msg( MSG_TTL_MENU_PET_SHOP_ASK_BUY,
 			ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
@@ -3741,6 +4151,7 @@ char	*cb_menu_pet_shop_buy_sel( menu_t **pp, menu_t **dflt )
 	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_BUY_SUCCESS, pet->name );
 
 	add_chr_gold( g_sel_mbr, (gold_t)(-price) );
+	call_game_sound_play( SOUND_KIND_SHOP_PAY, 1 );
 
 	return "/pet shop";
 }
@@ -3990,7 +4401,7 @@ bool_t	set_menu_auto_sale( menu_t *ls, mbr_t *mbr )
 
 char	*cb_menu_auto_sale_chk( menu_t **pp, menu_t **dflt )
 {
-	set_key_buf_str( "2" );
+	set_key_buf_str_head( "2" );
 
 	return STR_MENU_CANCEL;
 }
@@ -4036,6 +4447,7 @@ char	*cb_menu_auto_sale_exec( menu_t **pp, menu_t **dflt )
 		if( price > 0 )
 			add_chr_gold( g_sel_mbr, +price );
 	}
+	call_game_sound_play( SOUND_KIND_SHOP_PAY, 1 );
 
 	add_turn( NAVI_ADD_TURN_N + SHOP_ADD_TURN_N );
 
